# Generated by roxygen2: do not edit by hand

S3method("+",tmap)
S3method(knitr::knit_print,tmap)
S3method(knitr::knit_print,tmap_arrange)
S3method(print,tm_element)
S3method(print,tm_shape)
S3method(print,tmap)
S3method(print,tmap_arrange)
S3method(tmapGetShapeMeta1,Raster)
S3method(tmapGetShapeMeta1,SpatRaster)
S3method(tmapGetShapeMeta1,SpatVector)
S3method(tmapGetShapeMeta1,Spatial)
S3method(tmapGetShapeMeta1,default)
S3method(tmapGetShapeMeta1,sf)
S3method(tmapGetShapeMeta1,sfc)
S3method(tmapGetShapeMeta1,stars)
S3method(tmapGetShapeMeta2,SpatRaster)
S3method(tmapGetShapeMeta2,SpatVector)
S3method(tmapGetShapeMeta2,default)
S3method(tmapGetShapeMeta2,sf)
S3method(tmapGetShapeMeta2,sfc)
S3method(tmapGetShapeMeta2,stars)
S3method(tmapGridAuxPlot,default)
S3method(tmapGridAuxPlot,tm_aux_basemap)
S3method(tmapGridAuxPlot,tm_aux_graticules)
S3method(tmapGridAuxPlot,tm_aux_grid)
S3method(tmapGridAuxPlot,tm_aux_tiles)
S3method(tmapGridAuxPrepare,default)
S3method(tmapGridAuxPrepare,tm_aux_basemap)
S3method(tmapGridAuxPrepare,tm_aux_graticules)
S3method(tmapGridAuxPrepare,tm_aux_grid)
S3method(tmapGridAuxPrepare,tm_aux_tiles)
S3method(tmapGridCompHeight,tm_chart)
S3method(tmapGridCompHeight,tm_compass)
S3method(tmapGridCompHeight,tm_credits)
S3method(tmapGridCompHeight,tm_inset_grob)
S3method(tmapGridCompHeight,tm_inset_map)
S3method(tmapGridCompHeight,tm_legend_landscape)
S3method(tmapGridCompHeight,tm_legend_portrait)
S3method(tmapGridCompHeight,tm_logo)
S3method(tmapGridCompHeight,tm_minimap)
S3method(tmapGridCompHeight,tm_scalebar)
S3method(tmapGridCompHeight,tm_title)
S3method(tmapGridCompPlot,tm_chart_bar)
S3method(tmapGridCompPlot,tm_chart_box)
S3method(tmapGridCompPlot,tm_chart_donut)
S3method(tmapGridCompPlot,tm_chart_heatmap)
S3method(tmapGridCompPlot,tm_chart_histogram)
S3method(tmapGridCompPlot,tm_chart_violin)
S3method(tmapGridCompPlot,tm_compass)
S3method(tmapGridCompPlot,tm_credits)
S3method(tmapGridCompPlot,tm_inset_grob)
S3method(tmapGridCompPlot,tm_inset_map)
S3method(tmapGridCompPlot,tm_legend_landscape)
S3method(tmapGridCompPlot,tm_legend_portrait)
S3method(tmapGridCompPlot,tm_logo)
S3method(tmapGridCompPlot,tm_minimap)
S3method(tmapGridCompPlot,tm_scalebar)
S3method(tmapGridCompPlot,tm_title)
S3method(tmapGridCompPrepare,default)
S3method(tmapGridCompPrepare,tm_chart)
S3method(tmapGridCompPrepare,tm_chart_bar)
S3method(tmapGridCompPrepare,tm_chart_histogram)
S3method(tmapGridCompPrepare,tm_compass)
S3method(tmapGridCompPrepare,tm_credits)
S3method(tmapGridCompPrepare,tm_inset_gg)
S3method(tmapGridCompPrepare,tm_inset_grob)
S3method(tmapGridCompPrepare,tm_inset_image)
S3method(tmapGridCompPrepare,tm_inset_map)
S3method(tmapGridCompPrepare,tm_inset_tmap)
S3method(tmapGridCompPrepare,tm_legend_landscape)
S3method(tmapGridCompPrepare,tm_legend_portrait)
S3method(tmapGridCompPrepare,tm_logo)
S3method(tmapGridCompPrepare,tm_minimap)
S3method(tmapGridCompPrepare,tm_scalebar)
S3method(tmapGridCompPrepare,tm_title)
S3method(tmapGridCompWidth,tm_chart)
S3method(tmapGridCompWidth,tm_compass)
S3method(tmapGridCompWidth,tm_credits)
S3method(tmapGridCompWidth,tm_inset_grob)
S3method(tmapGridCompWidth,tm_inset_map)
S3method(tmapGridCompWidth,tm_legend_landscape)
S3method(tmapGridCompWidth,tm_legend_portrait)
S3method(tmapGridCompWidth,tm_logo)
S3method(tmapGridCompWidth,tm_minimap)
S3method(tmapGridCompWidth,tm_scalebar)
S3method(tmapGridCompWidth,tm_title)
S3method(tmapGridDataPlot,default)
S3method(tmapGridDataPlot,tm_data_borders)
S3method(tmapGridDataPlot,tm_data_bubbles)
S3method(tmapGridDataPlot,tm_data_dots)
S3method(tmapGridDataPlot,tm_data_fill)
S3method(tmapGridDataPlot,tm_data_iso)
S3method(tmapGridDataPlot,tm_data_labels)
S3method(tmapGridDataPlot,tm_data_labels_highlighted)
S3method(tmapGridDataPlot,tm_data_lines)
S3method(tmapGridDataPlot,tm_data_markers)
S3method(tmapGridDataPlot,tm_data_polygons)
S3method(tmapGridDataPlot,tm_data_raster)
S3method(tmapGridDataPlot,tm_data_squares)
S3method(tmapGridDataPlot,tm_data_symbols)
S3method(tmapGridDataPlot,tm_data_text)
S3method(tmapLeafletAuxPlot,tm_aux_basemap)
S3method(tmapLeafletAuxPlot,tm_aux_graticules)
S3method(tmapLeafletAuxPlot,tm_aux_grid)
S3method(tmapLeafletAuxPlot,tm_aux_tiles)
S3method(tmapLeafletAuxPrepare,tm_aux_basemap)
S3method(tmapLeafletAuxPrepare,tm_aux_graticules)
S3method(tmapLeafletAuxPrepare,tm_aux_grid)
S3method(tmapLeafletAuxPrepare,tm_aux_tiles)
S3method(tmapLeafletCompHeight,tm_compass)
S3method(tmapLeafletCompHeight,tm_credits)
S3method(tmapLeafletCompHeight,tm_legend_landscape)
S3method(tmapLeafletCompHeight,tm_legend_portrait)
S3method(tmapLeafletCompHeight,tm_logo)
S3method(tmapLeafletCompHeight,tm_minimap)
S3method(tmapLeafletCompHeight,tm_mouse_coordinates)
S3method(tmapLeafletCompHeight,tm_scalebar)
S3method(tmapLeafletCompHeight,tm_title)
S3method(tmapLeafletCompPlot,tm_compass)
S3method(tmapLeafletCompPlot,tm_credits)
S3method(tmapLeafletCompPlot,tm_legend_landscape)
S3method(tmapLeafletCompPlot,tm_legend_portrait)
S3method(tmapLeafletCompPlot,tm_logo)
S3method(tmapLeafletCompPlot,tm_minimap)
S3method(tmapLeafletCompPlot,tm_mouse_coordinates)
S3method(tmapLeafletCompPlot,tm_scalebar)
S3method(tmapLeafletCompPlot,tm_title)
S3method(tmapLeafletCompPrepare,default)
S3method(tmapLeafletCompPrepare,tm_chart_none)
S3method(tmapLeafletCompPrepare,tm_compass)
S3method(tmapLeafletCompPrepare,tm_credits)
S3method(tmapLeafletCompPrepare,tm_inset_gg)
S3method(tmapLeafletCompPrepare,tm_inset_grob)
S3method(tmapLeafletCompPrepare,tm_inset_image)
S3method(tmapLeafletCompPrepare,tm_legend_landscape)
S3method(tmapLeafletCompPrepare,tm_legend_portrait)
S3method(tmapLeafletCompPrepare,tm_logo)
S3method(tmapLeafletCompPrepare,tm_minimap)
S3method(tmapLeafletCompPrepare,tm_mouse_coordinates)
S3method(tmapLeafletCompPrepare,tm_scalebar)
S3method(tmapLeafletCompPrepare,tm_title)
S3method(tmapLeafletCompWidth,tm_compass)
S3method(tmapLeafletCompWidth,tm_credits)
S3method(tmapLeafletCompWidth,tm_legend_landscape)
S3method(tmapLeafletCompWidth,tm_legend_portrait)
S3method(tmapLeafletCompWidth,tm_logo)
S3method(tmapLeafletCompWidth,tm_minimap)
S3method(tmapLeafletCompWidth,tm_mouse_coordinates)
S3method(tmapLeafletCompWidth,tm_scalebar)
S3method(tmapLeafletCompWidth,tm_title)
S3method(tmapLeafletDataPlot,default)
S3method(tmapLeafletDataPlot,tm_data_borders)
S3method(tmapLeafletDataPlot,tm_data_bubbles)
S3method(tmapLeafletDataPlot,tm_data_dots)
S3method(tmapLeafletDataPlot,tm_data_fill)
S3method(tmapLeafletDataPlot,tm_data_iso)
S3method(tmapLeafletDataPlot,tm_data_labels)
S3method(tmapLeafletDataPlot,tm_data_labels_highlighted)
S3method(tmapLeafletDataPlot,tm_data_lines)
S3method(tmapLeafletDataPlot,tm_data_markers)
S3method(tmapLeafletDataPlot,tm_data_polygons)
S3method(tmapLeafletDataPlot,tm_data_raster)
S3method(tmapLeafletDataPlot,tm_data_squares)
S3method(tmapLeafletDataPlot,tm_data_symbols)
S3method(tmapLeafletDataPlot,tm_data_text)
S3method(tmapReproject,dimensions)
S3method(tmapReproject,sfc)
S3method(tmapReproject,stars)
S3method(tmapShape,Raster)
S3method(tmapShape,SpatRaster)
S3method(tmapShape,SpatVector)
S3method(tmapShape,Spatial)
S3method(tmapShape,default)
S3method(tmapShape,sf)
S3method(tmapShape,stars)
S3method(tmapSplitShp,default)
S3method(tmapSplitShp,stars)
S3method(tmapSubsetShp,Raster)
S3method(tmapSubsetShp,SpatRaster)
S3method(tmapSubsetShp,SpatVector)
S3method(tmapSubsetShp,Spatial)
S3method(tmapSubsetShp,default)
S3method(tmapSubsetShp,sf)
S3method(tmapSubsetShp,sfc)
S3method(tmapSubsetShp,stars)
export(.TMAP)
export(.TMAP_GRID)
export(.TMAP_LEAFLET)
export(.tmap_providers)
export(chart_save)
export(data_class)
export(data_type)
export(format_aes_results)
export(get_fact_levels_na)
export(get_scale_defaults)
export(lwd_to_mm)
export(make_by_vars)
export(marker_icon)
export(opt_tm_bubbles)
export(opt_tm_dots)
export(opt_tm_labels)
export(opt_tm_lines)
export(opt_tm_markers)
export(opt_tm_polygons)
export(opt_tm_raster)
export(opt_tm_rgb)
export(opt_tm_sf)
export(opt_tm_squares)
export(opt_tm_symbols)
export(opt_tm_text)
export(qtm)
export(renderTmap)
export(rtm)
export(rtmp)
export(shapeTM)
export(theme_ps)
export(tm_add_legend)
export(tm_animate)
export(tm_animate_fast)
export(tm_basemap)
export(tm_borders)
export(tm_bubbles)
export(tm_chart_bar)
export(tm_chart_box)
export(tm_chart_donut)
export(tm_chart_heatmap)
export(tm_chart_histogram)
export(tm_chart_none)
export(tm_chart_violin)
export(tm_check_fix)
export(tm_compass)
export(tm_components)
export(tm_const)
export(tm_credits)
export(tm_crs)
export(tm_dots)
export(tm_element)
export(tm_element_list)
export(tm_extra_inner_margin)
export(tm_facets)
export(tm_facets_flip)
export(tm_facets_grid)
export(tm_facets_hstack)
export(tm_facets_pagewise)
export(tm_facets_stack)
export(tm_facets_vstack)
export(tm_facets_wrap)
export(tm_fill)
export(tm_format)
export(tm_graticules)
export(tm_grid)
export(tm_group)
export(tm_inset)
export(tm_iso)
export(tm_label_format)
export(tm_labels)
export(tm_labels_highlighted)
export(tm_layout)
export(tm_legend)
export(tm_legend_bivariate)
export(tm_legend_combine)
export(tm_legend_hide)
export(tm_lines)
export(tm_logo)
export(tm_markers)
export(tm_minimap)
export(tm_mouse_coordinates)
export(tm_options)
export(tm_place_legends_bottom)
export(tm_place_legends_inside)
export(tm_place_legends_left)
export(tm_place_legends_right)
export(tm_place_legends_top)
export(tm_plot)
export(tm_plot_order)
export(tm_polygons)
export(tm_pos)
export(tm_pos_auto_in)
export(tm_pos_auto_out)
export(tm_pos_in)
export(tm_pos_on_top)
export(tm_pos_out)
export(tm_raster)
export(tm_remove_layer)
export(tm_rgb)
export(tm_rgba)
export(tm_scale)
export(tm_scale_asis)
export(tm_scale_bivariate)
export(tm_scale_categorical)
export(tm_scale_continuous)
export(tm_scale_continuous_log)
export(tm_scale_continuous_log10)
export(tm_scale_continuous_log1p)
export(tm_scale_continuous_log2)
export(tm_scale_continuous_pseudo_log)
export(tm_scale_continuous_sqrt)
export(tm_scale_discrete)
export(tm_scale_intervals)
export(tm_scale_ordinal)
export(tm_scale_rank)
export(tm_scale_rgb)
export(tm_scale_rgba)
export(tm_scalebar)
export(tm_seq)
export(tm_sf)
export(tm_shape)
export(tm_squares)
export(tm_style)
export(tm_symbols)
export(tm_text)
export(tm_tiles)
export(tm_title)
export(tm_title_in)
export(tm_title_out)
export(tm_vars)
export(tm_view)
export(tm_xlab)
export(tm_ylab)
export(tmapAddLayerOptions)
export(tmapChartBinned)
export(tmapChartBinned2d)
export(tmapChartBinned2d_catcat)
export(tmapChartBinned2d_numcat)
export(tmapChartBinned2d_numnum)
export(tmapChartBinned_categorical)
export(tmapChartBinned_numeric)
export(tmapChartNone)
export(tmapChartPass)
export(tmapChartRaw)
export(tmapChartRaw_nna)
export(tmapGetCompGroupArgs)
export(tmapGetShapeMeta1)
export(tmapGetShapeMeta2)
export(tmapGpar)
export(tmapGridAuxPlot)
export(tmapGridAuxPrepare)
export(tmapGridCompHeight)
export(tmapGridCompPlot)
export(tmapGridCompPrepare)
export(tmapGridCompWidth)
export(tmapGridDataPlot)
export(tmapLeafletAuxPlot)
export(tmapLeafletAuxPrepare)
export(tmapLeafletCompPrepare)
export(tmapLeafletDataPlot)
export(tmapMode)
export(tmapOutput)
export(tmapProxy)
export(tmapReproject)
export(tmapScale)
export(tmapScaleAsIs)
export(tmapScaleAuto)
export(tmapScaleBivariate)
export(tmapScaleCategorical)
export(tmapScaleIntervals)
export(tmapScaleRank)
export(tmapSeq)
export(tmapShape)
export(tmapSplitShp)
export(tmapSubmitOptions)
export(tmapSubsetShp)
export(tmapTpar)
export(tmapTransCentroid)
export(tmapTransLines)
export(tmapTransPolygons)
export(tmapTransRaster)
export(tmapUsrCls)
export(tmapValuesBVV_bgcol)
export(tmapValuesBVV_col)
export(tmapValuesBVV_fill)
export(tmapValuesCVV_angle)
export(tmapValuesCVV_area)
export(tmapValuesCVV_bgcol)
export(tmapValuesCVV_bgcol_alpha)
export(tmapValuesCVV_col)
export(tmapValuesCVV_col_alpha)
export(tmapValuesCVV_fill)
export(tmapValuesCVV_fill_alpha)
export(tmapValuesCVV_fontface)
export(tmapValuesCVV_lty)
export(tmapValuesCVV_lwd)
export(tmapValuesCVV_num)
export(tmapValuesCVV_shape)
export(tmapValuesCVV_size)
export(tmapValuesCVV_skip)
export(tmapValuesCVV_text)
export(tmapValuesCVV_xmod)
export(tmapValuesCVV_ymod)
export(tmapValuesCheck_angle)
export(tmapValuesCheck_area)
export(tmapValuesCheck_bgcol)
export(tmapValuesCheck_bgcol_alpha)
export(tmapValuesCheck_col)
export(tmapValuesCheck_col_alpha)
export(tmapValuesCheck_fill)
export(tmapValuesCheck_fill_alpha)
export(tmapValuesCheck_fontface)
export(tmapValuesCheck_lty)
export(tmapValuesCheck_lwd)
export(tmapValuesCheck_num)
export(tmapValuesCheck_shape)
export(tmapValuesCheck_size)
export(tmapValuesCheck_skip)
export(tmapValuesCheck_text)
export(tmapValuesCheck_xmod)
export(tmapValuesColorize_angle)
export(tmapValuesColorize_area)
export(tmapValuesColorize_bgcol)
export(tmapValuesColorize_bgcol_alpha)
export(tmapValuesColorize_col)
export(tmapValuesColorize_col_alpha)
export(tmapValuesColorize_fill)
export(tmapValuesColorize_fill_alpha)
export(tmapValuesColorize_fontface)
export(tmapValuesColorize_lty)
export(tmapValuesColorize_lwd)
export(tmapValuesColorize_num)
export(tmapValuesColorize_shape)
export(tmapValuesColorize_size)
export(tmapValuesColorize_skip)
export(tmapValuesColorize_text)
export(tmapValuesColorize_xmod)
export(tmapValuesIsDiv_angle)
export(tmapValuesIsDiv_area)
export(tmapValuesIsDiv_bgcol)
export(tmapValuesIsDiv_bgcol_alpha)
export(tmapValuesIsDiv_col)
export(tmapValuesIsDiv_col_alpha)
export(tmapValuesIsDiv_fill)
export(tmapValuesIsDiv_fill_alpha)
export(tmapValuesIsDiv_fontface)
export(tmapValuesIsDiv_lty)
export(tmapValuesIsDiv_lwd)
export(tmapValuesIsDiv_num)
export(tmapValuesIsDiv_shape)
export(tmapValuesIsDiv_size)
export(tmapValuesIsDiv_skip)
export(tmapValuesIsDiv_text)
export(tmapValuesIsDiv_xmod)
export(tmapValuesRange_angle)
export(tmapValuesRange_area)
export(tmapValuesRange_bgcol)
export(tmapValuesRange_bgcol_alpha)
export(tmapValuesRange_col)
export(tmapValuesRange_col_alpha)
export(tmapValuesRange_fill)
export(tmapValuesRange_fill_alpha)
export(tmapValuesRange_fontface)
export(tmapValuesRange_lty)
export(tmapValuesRange_lwd)
export(tmapValuesRange_num)
export(tmapValuesRange_shape)
export(tmapValuesRange_size)
export(tmapValuesRange_skip)
export(tmapValuesRange_text)
export(tmapValuesRange_xmod)
export(tmapValuesScale_angle)
export(tmapValuesScale_area)
export(tmapValuesScale_bgcol)
export(tmapValuesScale_bgcol_alpha)
export(tmapValuesScale_col)
export(tmapValuesScale_col_alpha)
export(tmapValuesScale_fill)
export(tmapValuesScale_fill_alpha)
export(tmapValuesScale_fontface)
export(tmapValuesScale_lty)
export(tmapValuesScale_lwd)
export(tmapValuesScale_num)
export(tmapValuesScale_shape)
export(tmapValuesScale_size)
export(tmapValuesScale_skip)
export(tmapValuesScale_text)
export(tmapValuesScale_xmod)
export(tmapValuesSubmit_angle)
export(tmapValuesSubmit_area)
export(tmapValuesSubmit_bgcol)
export(tmapValuesSubmit_bgcol_alpha)
export(tmapValuesSubmit_col)
export(tmapValuesSubmit_col_alpha)
export(tmapValuesSubmit_fill)
export(tmapValuesSubmit_fill_alpha)
export(tmapValuesSubmit_fontface)
export(tmapValuesSubmit_lty)
export(tmapValuesSubmit_lwd)
export(tmapValuesSubmit_num)
export(tmapValuesSubmit_shape)
export(tmapValuesSubmit_size)
export(tmapValuesSubmit_skip)
export(tmapValuesSubmit_text)
export(tmapValuesSubmit_xmod)
export(tmapValuesSubmit_ymod)
export(tmapValuesVV_angle)
export(tmapValuesVV_area)
export(tmapValuesVV_bgcol)
export(tmapValuesVV_bgcol_alpha)
export(tmapValuesVV_col)
export(tmapValuesVV_col_alpha)
export(tmapValuesVV_fill)
export(tmapValuesVV_fill_alpha)
export(tmapValuesVV_fontface)
export(tmapValuesVV_lty)
export(tmapValuesVV_lwd)
export(tmapValuesVV_num)
export(tmapValuesVV_shape)
export(tmapValuesVV_size)
export(tmapValuesVV_skip)
export(tmapValuesVV_text)
export(tmapValuesVV_xmod)
export(tmap_animation)
export(tmap_arrange)
export(tmap_design_mode)
export(tmap_devel_mode)
export(tmap_format)
export(tmap_format_add)
export(tmap_grob)
export(tmap_icons)
export(tmap_last)
export(tmap_leaflet)
export(tmap_mode)
export(tmap_options)
export(tmap_options_diff)
export(tmap_options_mode)
export(tmap_options_reset)
export(tmap_options_save)
export(tmap_overview)
export(tmap_provider_credits)
export(tmap_providers)
export(tmap_save)
export(tmap_style)
export(tmap_style_catalog)
export(tmap_style_catalogue)
export(tmap_tip)
export(toTitleCase)
export(transform_values)
export(ttm)
export(ttmp)
import(classInt)
import(cols4all)
import(data.table)
import(grid)
import(htmltools)
import(htmlwidgets)
import(leafem)
import(leaflegend)
import(leaflet)
import(leafsync)
import(stats)
import(tmaptools)
import(units)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.size)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(htmlwidgets,saveWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(leaflet,icons)
importFrom(methods,as)
importFrom(rlang,expr)
importFrom(rlang,missing_arg)
importFrom(s2,as_s2_geography)
importFrom(s2,s2_buffer_cells)
importFrom(s2,s2_intersects)
importFrom(sf,st_as_sf)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_sf)
importFrom(sf,st_transform)
importFrom(stars,st_as_stars)
importFrom(stars,st_dimensions)
importFrom(stars,st_get_dimension_values)
importFrom(stars,st_set_dimensions)
importFrom(stars,st_warp)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,getS3method)
importFrom(utils,head)
importFrom(utils,methods)
importFrom(utils,setTxtProgressBar)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
