scale_init = function() {
	assign("scls", list(), envir = .TMAP)
}

scale_save = function(scale) {
	if (!exists("scls", envir = .TMAP)) scale_init()
	scls = get("scls", envir = .TMAP)
	scls = c(scls, (list(scale)))
	assign("scls", scls, envir = .TMAP)
	length(scls)
}

scale_load = function(nr) {
	if (!exists("scls", envir = .TMAP)) return(NULL)
	scls = get("scls", envir = .TMAP)
	scls[[nr]]
}
