% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tm_scale_.R
\name{tm_const}
\alias{tm_const}
\title{tmap function to define a constant visual value}
\usage{
tm_const()
}
\description{
tmap function to define a constant visual value
}
