% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tmap_tip.R
\name{tmap_tip}
\alias{tmap_tip}
\title{Print a random tip to the console}
\usage{
tmap_tip()
}
\value{
A message
}
\description{
Print a random tip to the console
}
