% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tm_options.R
\name{tm_options}
\alias{tm_options}
\title{tmap options}
\usage{
tm_options(
  crs,
  facet.max,
  facet_levels.max,
  free.scales,
  raster.max_cells,
  raster.warp,
  show.messages,
  show.warnings,
  output.format,
  output.size,
  output.dpi,
  animation.dpi,
  value.const,
  value.na,
  value.null,
  value.blank,
  values.var,
  values.range,
  value.neutral,
  values.scale,
  scales.var,
  scale.misc.args,
  continuous.nclass_per_legend_break,
  continuous.nclasses,
  label.format,
  label.na,
  scale,
  asp,
  bg,
  bg.color,
  outer.bg,
  outer.bg.color,
  frame,
  frame.color,
  frame.alpha,
  frame.lwd,
  frame.r,
  frame.double_line,
  outer.margins,
  inner.margins,
  inner.margins.extra,
  meta.margins,
  meta.auto_margins,
  between_margin,
  panel.margin,
  xlab.show,
  xlab.text,
  xlab.size,
  xlab.color,
  xlab.rotation,
  xlab.space,
  xlab.fontface,
  xlab.fontfamily,
  xlab.alpha,
  xlab.side,
  ylab.show,
  ylab.text,
  ylab.size,
  ylab.color,
  ylab.rotation,
  ylab.space,
  ylab.fontface,
  ylab.fontfamily,
  ylab.alpha,
  ylab.side,
  panel.type,
  panel.wrap.pos,
  panel.xtab.pos,
  unit,
  color.sepia_intensity,
  color.saturation,
  color_vision_deficiency_sim,
  text.fontface,
  text.fontfamily,
  r,
  component.position,
  component.offset,
  component.stack_margin,
  component.autoscale,
  component.resize_as_group,
  component.frame_combine,
  component.stack,
  legend.stack,
  chart.stack,
  component.equalize,
  component.frame,
  component.frame.color,
  component.frame.alpha,
  component.frame.lwd,
  component.frame.r,
  component.bg,
  component.bg.color,
  component.bg.alpha,
  legend.show,
  legend.orientation,
  legend.position,
  legend.width,
  legend.height,
  legend.reverse,
  legend.na.show,
  legend.title.color,
  legend.title.size,
  legend.title.fontface,
  legend.title.fontfamily,
  legend.title.alpha,
  legend.xlab.color,
  legend.xlab.size,
  legend.xlab.rot,
  legend.xlab.fontface,
  legend.xlab.fontfamily,
  legend.xlab.alpha,
  legend.ylab.color,
  legend.ylab.size,
  legend.ylab.rot,
  legend.ylab.fontface,
  legend.ylab.fontfamily,
  legend.ylab.alpha,
  legend.text.color,
  legend.text.size,
  legend.text.fontface,
  legend.text.fontfamily,
  legend.text.alpha,
  legend.frame,
  legend.frame.color,
  legend.frame.alpha,
  legend.frame.lwd,
  legend.frame.r,
  legend.bg,
  legend.bg.color,
  legend.bg.alpha,
  legend.only,
  legend.absolute_fontsize,
  legend.settings.portrait,
  legend.settings.landscape,
  add_legend.position,
  chart.show,
  chart.plot.axis.x,
  chart.plot.axis.y,
  chart.position,
  chart.width,
  chart.height,
  chart.reverse,
  chart.na.show,
  chart.title.color,
  chart.title.size,
  chart.title.fontface,
  chart.title.fontfamily,
  chart.title.alpha,
  chart.xlab.color,
  chart.xlab.size,
  chart.xlab.fontface,
  chart.xlab.fontfamily,
  chart.xlab.alpha,
  chart.ylab.color,
  chart.ylab.size,
  chart.ylab.fontface,
  chart.ylab.fontfamily,
  chart.ylab.alpha,
  chart.text.color,
  chart.text.size,
  chart.text.fontface,
  chart.text.fontfamily,
  chart.text.alpha,
  chart.frame,
  chart.frame.color,
  chart.frame.alpha,
  chart.frame.lwd,
  chart.frame.r,
  chart.bg,
  chart.bg.color,
  chart.bg.alpha,
  chart.object.color,
  title.size,
  title.color,
  title.fontface,
  title.fontfamily,
  title.alpha,
  title.padding,
  title.frame,
  title.frame.color,
  title.frame.alpha,
  title.frame.lwd,
  title.frame.r,
  title.position,
  title.width,
  credits.size,
  credits.color,
  credits.fontface,
  credits.fontfamily,
  credits.alpha,
  credits.padding,
  credits.position,
  credits.width,
  credits.height,
  compass.north,
  compass.type,
  compass.text.size,
  compass.size,
  compass.show.labels,
  compass.cardinal.directions,
  compass.text.color,
  compass.color.dark,
  compass.color.light,
  compass.lwd,
  compass.margins,
  compass.position,
  inset.position,
  logo.height,
  logo.margins,
  logo.between_margin,
  logo.position,
  inset_map.height,
  inset_map.width,
  inset_map.margins,
  inset_map.between_margin,
  inset_map.position,
  inset_map.frame,
  inset.height,
  inset.width,
  inset.margins,
  inset.between_margin,
  inset.frame,
  inset.bg,
  inset.bg.color,
  inset.bg.alpha,
  inset_grob.height,
  inset_grob.width,
  inset_gg.height,
  inset_gg.width,
  scalebar.breaks,
  scalebar.width,
  scalebar.allow_clipping,
  scalebar.text.size,
  scalebar.text.color,
  scalebar.text.fontface,
  scalebar.text.fontfamily,
  scalebar.color.dark,
  scalebar.color.light,
  scalebar.lwd,
  scalebar.size,
  scalebar.margins,
  scalebar.position,
  grid.show,
  grid.labels.pos,
  grid.x,
  grid.y,
  grid.n.x,
  grid.n.y,
  grid.crs,
  grid.col,
  grid.lwd,
  grid.alpha,
  grid.labels.show,
  grid.labels.size,
  grid.labels.col,
  grid.labels.fontface,
  grid.labels.fontfamily,
  grid.labels.rot,
  grid.labels.format,
  grid.labels.cardinal,
  grid.labels.margin.x,
  grid.labels.margin.y,
  grid.labels.space.x,
  grid.labels.space.y,
  grid.labels.inside_frame,
  grid.ticks,
  grid.lines,
  grid.ndiscr,
  mouse_coordinates.position,
  minimap.server,
  minimap.toggle,
  minimap.position,
  panel.show,
  panel.labels,
  panel.label.size,
  panel.label.color,
  panel.label.fontface,
  panel.label.fontfamily,
  panel.label.alpha,
  panel.label.bg,
  panel.label.bg.color,
  panel.label.bg.alpha,
  panel.label.frame,
  panel.label.frame.color,
  panel.label.frame.alpha,
  panel.label.frame.lwd,
  panel.label.frame.r,
  panel.label.height,
  panel.label.rot,
  qtm.scalebar,
  qtm.minimap,
  qtm.mouse_coordinates,
  earth_boundary,
  earth_boundary.color,
  earth_boundary.lwd,
  earth_datum,
  space,
  space.color,
  space_overlay,
  check_and_fix,
  basemap.show,
  basemap.server,
  basemap.alpha,
  basemap.zoom,
  tiles.show,
  tiles.server,
  tiles.alpha,
  tiles.zoom,
  attr.color,
  crs_extra,
  crs_global,
  crs_basemap,
  title = NULL,
  main.title = NULL,
  main.title.size = NULL,
  main.title.color = NULL,
  main.title.fontface = NULL,
  main.title.fontfamily = NULL,
  main.title.position = NULL,
  fontface = NULL,
  fontfamily = NULL,
  style,
  ...
)
}
\arguments{
\item{crs}{Map crs (see \code{\link[=tm_shape]{tm_shape()}}). \code{NA} means the crs is specified in \code{\link[=tm_shape]{tm_shape()}}. The crs that is used by the transformation functions is defined in \code{\link[=tm_shape]{tm_shape()}}.}

\item{facet.max}{Maximum number of facets in a static plot (rows x columns); animation frames are not counted.}

\item{facet_levels.max}{Maximum number of facet levels processed across all faceting dimensions, animation frames included (rows x columns x frames). Default \code{1e5}.}

\item{free.scales}{For backward compatibility: if this value is set, it will be used to impute the free arguments in the layer functions}

\item{raster.max_cells}{Maximum number of raster grid cells. Can be mode specific \code{c(plot = 3000, view = 1000, 1000)} (the last value is the fall back default)}

\item{raster.warp}{Should rasters be warped or transformed in case a different projection (crs) is used? Warping creates a new regular raster in the target crs, whereas transforming creates a (usually non-regular) raster in the target crs. The former is lossy, but much faster and is therefore the default.
When a different projection (crs) is used, a (usually) regular raster will be}

\item{show.messages}{Show messages?}

\item{show.warnings}{Show warnings?}

\item{output.format}{Output format}

\item{output.size}{Output size}

\item{output.dpi}{Output dpi}

\item{animation.dpi}{Output dpi for animations}

\item{value.const}{Default visual value constants e.g. the default fill color for \code{tm_shape(World) + tm_polygons()}. A list is required with per visual variable a value.}

\item{value.na}{Default visual values that are used to visualize NA data values. A list is required with per visual variable a value.}

\item{value.null}{Default visual values that are used to visualize null (out-of-scope) data values. A list is required with per visual variable a value.}

\item{value.blank}{Default visual values that correspond to blank. For color these are \code{"#00000000"} meaning transparent. A list is required with per visual variable a value.}

\item{values.var}{Default values when a data variable to mapped to a visual variable, e.g. a color palette. A list is required with per visual variable a value.}

\item{values.range}{Default range for values. See \code{values.range} of \code{\link[=tm_scale_categorical]{tm_scale_categorical()}}. A list is required with per visual variable a value.}

\item{value.neutral}{Default values for when a data variable to mapped to a visual variable, e.g. a color palette. A list is required with per visual variable a value.}

\item{values.scale}{Default scales (as in object sizes) for values. See \code{values.range} of \code{\link[=tm_scale_categorical]{tm_scale_categorical()}}. A list is required with per visual variable a value.}

\item{scales.var}{Default scale functions per visual variable and type of data variable. A list is required with per visual variable per data type.}

\item{scale.misc.args}{Default values of scale function-specific arguments. A list is required with per scale function and optional per visual variable.}

\item{continuous.nclass_per_legend_break}{The number of continuous legend breaks within one 'unit' (label).  The default value is 50.}

\item{continuous.nclasses}{the number of classes of a continuous scale. Should be odd.  The default value is 101.}

\item{label.format}{Format for the labels. These are the default values for \code{\link[=tm_label_format]{tm_label_format()}}}

\item{label.na}{Default label for missing values.}

\item{scale}{Overall scale of the map}

\item{asp}{Aspect ratio of each map. When \code{asp} is set to \code{NA} (default) the aspect ratio will be adjusted to the used shapes. When set to 0 the aspect ratio is
adjusted to the size of the device divided by the number of columns and rows.}

\item{bg}{Draw map background?}

\item{bg.color}{Background color of the map.}

\item{outer.bg}{Draw map background (outside the frame)?}

\item{outer.bg.color}{Background color of map outside the frame.}

\item{frame}{Draw map frame?}

\item{frame.color}{The color of the frame.}

\item{frame.alpha}{The alpha transparency of the frame.}

\item{frame.lwd}{The line width of the frame. See \code{graphics::par}, option 'lwd'.}

\item{frame.r}{The r (radius) of the frame.}

\item{frame.double_line}{The double line of the frame. \code{TRUE} or \code{FALSE}.}

\item{outer.margins}{The margins of the outer space (outside the frame. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{inner.margins}{The margins of the inner space (inside the frame). A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{inner.margins.extra}{The extra arguments of the margins of the inner space (inside the frame). A list of arguments.}

\item{meta.margins}{The margins of the 'meta' space (reserved for outside map components). A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{meta.auto_margins}{The auto_margins of the 'meta' space (reserved for outside map components).}

\item{between_margin}{Margin between the map.}

\item{panel.margin}{The margin of the panel.}

\item{xlab.show}{The visibility of the xlab. \code{TRUE} or \code{FALSE}.}

\item{xlab.text}{The text of the xlab.}

\item{xlab.size}{The size of the xlab.}

\item{xlab.color}{The color of the xlab.}

\item{xlab.rotation}{The rotation of the xlab.}

\item{xlab.space}{The space of the xlab. In terms of number of text line heights.}

\item{xlab.fontface}{The font face of the xlab. See \code{graphics::par}, option 'font'.}

\item{xlab.fontfamily}{The font family of the xlab. See \code{graphics::par}, option 'family'.}

\item{xlab.alpha}{The alpha transparency of the xlab.}

\item{xlab.side}{The side of the xlab.}

\item{ylab.show}{The visibility of the ylab. \code{TRUE} or \code{FALSE}.}

\item{ylab.text}{The text of the ylab.}

\item{ylab.size}{The size of the ylab.}

\item{ylab.color}{The color of the ylab.}

\item{ylab.rotation}{The rotation of the ylab.}

\item{ylab.space}{The space of the ylab. In terms of number of text line heights.}

\item{ylab.fontface}{The font face of the ylab. See \code{graphics::par}, option 'font'.}

\item{ylab.fontfamily}{The font family of the ylab. See \code{graphics::par}, option 'family'.}

\item{ylab.alpha}{The alpha transparency of the ylab.}

\item{ylab.side}{The side of the ylab.}

\item{panel.type}{The type of the panel.}

\item{panel.wrap.pos}{The panel positions for wrapped facets created with \code{\link[=tm_facets_grid]{tm_facets_grid()}}. One of \code{"left"}, \code{"right"}, \code{"top"} (default) or \code{"bottom"}.}

\item{panel.xtab.pos}{The panel positions for grid facets created with \code{\link[=tm_facets_grid]{tm_facets_grid()}}. Vector of two, where the first determines the locations of row panels (\code{"left"} or \code{"right"}) and the second the location of column panels ( \code{"top"} or `"bottom")}

\item{unit}{Unit of the coordinate}

\item{color.sepia_intensity}{The sepia_intensity of the color.}

\item{color.saturation}{The saturation of the color.}

\item{color_vision_deficiency_sim}{Color vision deficiency simulation. Either \code{"protan"}, \code{"deutan"}, or \code{"tritan"}.}

\item{text.fontface}{The font face of the text. See \code{graphics::par}, option 'font'.}

\item{text.fontfamily}{The font family of the text. See \code{graphics::par}, option 'family'.}

\item{r}{The r (radius) (overall).}

\item{component.position}{The position of the component. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{component.offset}{The offset of the component.}

\item{component.stack_margin}{The stack_margin of the component.}

\item{component.autoscale}{The autoscale of the component.}

\item{component.resize_as_group}{The resize_as_group of the component.}

\item{component.frame_combine}{The frame_combine of the component.}

\item{component.stack}{The stack of the component.}

\item{legend.stack}{The stack of the legend.}

\item{chart.stack}{The stack of the chart.}

\item{component.equalize}{The equalize of the component.}

\item{component.frame}{The frame of the component.}

\item{component.frame.color}{The color of the frame of the component.}

\item{component.frame.alpha}{The alpha transparency of the frame of the component.}

\item{component.frame.lwd}{The line width of the frame of the component. See \code{graphics::par}, option 'lwd'.}

\item{component.frame.r}{The r (radius) of the frame of the component.}

\item{component.bg}{The bg of the component.}

\item{component.bg.color}{The color of the bg of the component.}

\item{component.bg.alpha}{The alpha transparency of the bg of the component.}

\item{legend.show}{The visibility of the legend. \code{TRUE} or \code{FALSE}.}

\item{legend.orientation}{The orientation of the legend.}

\item{legend.position}{The position of the legend. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{legend.width}{The width of the legend.}

\item{legend.height}{The height of the legend.}

\item{legend.reverse}{The reverse of the legend.}

\item{legend.na.show}{The visibility of the na of the legend. \code{TRUE} or \code{FALSE}.}

\item{legend.title.color}{The color of the title of the legend.}

\item{legend.title.size}{The size of the title of the legend.}

\item{legend.title.fontface}{The font face of the title of the legend. See \code{graphics::par}, option 'font'.}

\item{legend.title.fontfamily}{The font family of the title of the legend. See \code{graphics::par}, option 'family'.}

\item{legend.title.alpha}{The alpha transparency of the title of the legend.}

\item{legend.xlab.color}{The color of the xlab of the legend.}

\item{legend.xlab.size}{The size of the xlab of the legend.}

\item{legend.xlab.rot}{The rot of the xlab of the legend.}

\item{legend.xlab.fontface}{The font face of the xlab of the legend. See \code{graphics::par}, option 'font'.}

\item{legend.xlab.fontfamily}{The font family of the xlab of the legend. See \code{graphics::par}, option 'family'.}

\item{legend.xlab.alpha}{The alpha transparency of the xlab of the legend.}

\item{legend.ylab.color}{The color of the ylab of the legend.}

\item{legend.ylab.size}{The size of the ylab of the legend.}

\item{legend.ylab.rot}{The rot of the ylab of the legend.}

\item{legend.ylab.fontface}{The font face of the ylab of the legend. See \code{graphics::par}, option 'font'.}

\item{legend.ylab.fontfamily}{The font family of the ylab of the legend. See \code{graphics::par}, option 'family'.}

\item{legend.ylab.alpha}{The alpha transparency of the ylab of the legend.}

\item{legend.text.color}{The color of the text of the legend.}

\item{legend.text.size}{The size of the text of the legend.}

\item{legend.text.fontface}{The font face of the text of the legend. See \code{graphics::par}, option 'font'.}

\item{legend.text.fontfamily}{The font family of the text of the legend. See \code{graphics::par}, option 'family'.}

\item{legend.text.alpha}{The alpha transparency of the text of the legend.}

\item{legend.frame}{The frame of the legend.}

\item{legend.frame.color}{The color of the frame of the legend.}

\item{legend.frame.alpha}{The alpha transparency of the frame of the legend.}

\item{legend.frame.lwd}{The line width of the frame of the legend. See \code{graphics::par}, option 'lwd'.}

\item{legend.frame.r}{The r (radius) of the frame of the legend.}

\item{legend.bg}{The bg of the legend.}

\item{legend.bg.color}{The color of the bg of the legend.}

\item{legend.bg.alpha}{The alpha transparency of the bg of the legend.}

\item{legend.only}{Should only legends be printed (so without map)?}

\item{legend.absolute_fontsize}{The absolute fontsize of the legend. So far, only used to calculate legend dimensions}

\item{legend.settings.portrait}{The portrait of the settings of the legend.}

\item{legend.settings.landscape}{The landscape of the settings of the legend.}

\item{add_legend.position}{The position of the add_legend. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{chart.show}{The visibility of the chart. \code{TRUE} or \code{FALSE}.}

\item{chart.plot.axis.x}{The x of the axis of the plot of the chart.}

\item{chart.plot.axis.y}{The y of the axis of the plot of the chart.}

\item{chart.position}{The position of the chart. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{chart.width}{The width of the chart.}

\item{chart.height}{The height of the chart.}

\item{chart.reverse}{The reverse of the chart.}

\item{chart.na.show}{The visibility of the na of the chart. \code{TRUE} or \code{FALSE}.}

\item{chart.title.color}{The color of the title of the chart.}

\item{chart.title.size}{The size of the title of the chart.}

\item{chart.title.fontface}{The font face of the title of the chart. See \code{graphics::par}, option 'font'.}

\item{chart.title.fontfamily}{The font family of the title of the chart. See \code{graphics::par}, option 'family'.}

\item{chart.title.alpha}{The alpha transparency of the title of the chart.}

\item{chart.xlab.color}{The color of the xlab of the chart.}

\item{chart.xlab.size}{The size of the xlab of the chart.}

\item{chart.xlab.fontface}{The font face of the xlab of the chart. See \code{graphics::par}, option 'font'.}

\item{chart.xlab.fontfamily}{The font family of the xlab of the chart. See \code{graphics::par}, option 'family'.}

\item{chart.xlab.alpha}{The alpha transparency of the xlab of the chart.}

\item{chart.ylab.color}{The color of the ylab of the chart.}

\item{chart.ylab.size}{The size of the ylab of the chart.}

\item{chart.ylab.fontface}{The font face of the ylab of the chart. See \code{graphics::par}, option 'font'.}

\item{chart.ylab.fontfamily}{The font family of the ylab of the chart. See \code{graphics::par}, option 'family'.}

\item{chart.ylab.alpha}{The alpha transparency of the ylab of the chart.}

\item{chart.text.color}{The color of the text of the chart.}

\item{chart.text.size}{The size of the text of the chart.}

\item{chart.text.fontface}{The font face of the text of the chart. See \code{graphics::par}, option 'font'.}

\item{chart.text.fontfamily}{The font family of the text of the chart. See \code{graphics::par}, option 'family'.}

\item{chart.text.alpha}{The alpha transparency of the text of the chart.}

\item{chart.frame}{The frame of the chart.}

\item{chart.frame.color}{The color of the frame of the chart.}

\item{chart.frame.alpha}{The alpha transparency of the frame of the chart.}

\item{chart.frame.lwd}{The line width of the frame of the chart. See \code{graphics::par}, option 'lwd'.}

\item{chart.frame.r}{The r (radius) of the frame of the chart.}

\item{chart.bg}{The bg of the chart.}

\item{chart.bg.color}{The color of the bg of the chart.}

\item{chart.bg.alpha}{The alpha transparency of the bg of the chart.}

\item{chart.object.color}{The color of the object of the chart.}

\item{title.size}{The size of the title.}

\item{title.color}{The color of the title.}

\item{title.fontface}{The font face of the title. See \code{graphics::par}, option 'font'.}

\item{title.fontfamily}{The font family of the title. See \code{graphics::par}, option 'family'.}

\item{title.alpha}{The alpha transparency of the title.}

\item{title.padding}{The padding of the title. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{title.frame}{The frame of the title.}

\item{title.frame.color}{The color of the frame of the title.}

\item{title.frame.alpha}{The alpha transparency of the frame of the title.}

\item{title.frame.lwd}{The line width of the frame of the title. See \code{graphics::par}, option 'lwd'.}

\item{title.frame.r}{The r (radius) of the frame of the title.}

\item{title.position}{The position of the title. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{title.width}{The width of the title.}

\item{credits.size}{The size of the credits.}

\item{credits.color}{The color of the credits.}

\item{credits.fontface}{The font face of the credits. See \code{graphics::par}, option 'font'.}

\item{credits.fontfamily}{The font family of the credits. See \code{graphics::par}, option 'family'.}

\item{credits.alpha}{The alpha transparency of the credits.}

\item{credits.padding}{The padding of the credits. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{credits.position}{The position of the credits. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{credits.width}{The width of the credits.}

\item{credits.height}{The height of the credits.}

\item{compass.north}{The north of the compass.}

\item{compass.type}{The type of the compass.}

\item{compass.text.size}{The size of the text of the compass.}

\item{compass.size}{The size of the compass.}

\item{compass.show.labels}{The labels of the show of the compass.}

\item{compass.cardinal.directions}{The directions of the cardinal of the compass.}

\item{compass.text.color}{The color of the text of the compass.}

\item{compass.color.dark}{The dark of the color of the compass.}

\item{compass.color.light}{The light of the color of the compass.}

\item{compass.lwd}{The line width of the compass. See \code{graphics::par}, option 'lwd'.}

\item{compass.margins}{The margins of the compass. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{compass.position}{The position of the compass. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{inset.position}{The position of the inset. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{logo.height}{The height of the logo.}

\item{logo.margins}{The margins of the logo. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{logo.between_margin}{The between_margin of the logo.}

\item{logo.position}{The position of the logo. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{inset_map.height}{The height of the inset_map.}

\item{inset_map.width}{The width of the inset_map.}

\item{inset_map.margins}{The margins of the inset_map. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{inset_map.between_margin}{The between_margin of the inset_map.}

\item{inset_map.position}{The position of the inset_map. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{inset_map.frame}{The frame of the inset_map.}

\item{inset.height}{The height of the inset.}

\item{inset.width}{The width of the inset.}

\item{inset.margins}{The margins of the inset. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{inset.between_margin}{The between_margin of the inset.}

\item{inset.frame}{The frame of the inset.}

\item{inset.bg}{The bg of the inset.}

\item{inset.bg.color}{The color of the bg of the inset.}

\item{inset.bg.alpha}{The alpha transparency of the bg of the inset.}

\item{inset_grob.height}{The height of the inset_grob.}

\item{inset_grob.width}{The width of the inset_grob.}

\item{inset_gg.height}{The height of the inset_gg.}

\item{inset_gg.width}{The width of the inset_gg.}

\item{scalebar.breaks}{See \code{\link[=tm_scalebar]{tm_scalebar()}}}

\item{scalebar.width}{See \code{\link[=tm_scalebar]{tm_scalebar()}}}

\item{scalebar.allow_clipping}{See \code{\link[=tm_scalebar]{tm_scalebar()}}}

\item{scalebar.text.size}{The size of the text of the scalebar.}

\item{scalebar.text.color}{The color of the text of the scalebar.}

\item{scalebar.text.fontface}{The font face of the text of the scalebar. See \code{graphics::par}, option 'font'.}

\item{scalebar.text.fontfamily}{The font family of the text of the scalebar. See \code{graphics::par}, option 'family'.}

\item{scalebar.color.dark}{The dark of the color of the scalebar.}

\item{scalebar.color.light}{The light of the color of the scalebar.}

\item{scalebar.lwd}{The line width of the scalebar. See \code{graphics::par}, option 'lwd'.}

\item{scalebar.size}{The size of the scalebar.}

\item{scalebar.margins}{The margins of the scalebar. A vector of 4 values: bottom, left, top, right. The unit is the device height (for bottom and top) or width (for left and right).}

\item{scalebar.position}{The position of the scalebar. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{grid.show}{The visibility of the grid. \code{TRUE} or \code{FALSE}.}

\item{grid.labels.pos}{The pos of the labels of the grid.}

\item{grid.x}{The x of the grid.}

\item{grid.y}{The y of the grid.}

\item{grid.n.x}{The x of the n of the grid.}

\item{grid.n.y}{The y of the n of the grid.}

\item{grid.crs}{The coordinate reference system (CRS) of the grid.}

\item{grid.col}{The color of the grid.}

\item{grid.lwd}{The line width of the grid. See \code{graphics::par}, option 'lwd'.}

\item{grid.alpha}{The alpha transparency of the grid.}

\item{grid.labels.show}{The visibility of the labels of the grid. \code{TRUE} or \code{FALSE}.}

\item{grid.labels.size}{The size of the labels of the grid.}

\item{grid.labels.col}{The color of the labels of the grid.}

\item{grid.labels.fontface}{The font face of the labels of the grid. See \code{graphics::par}, option 'font'.}

\item{grid.labels.fontfamily}{The font family of the labels of the grid. See \code{graphics::par}, option 'family'.}

\item{grid.labels.rot}{The rot of the labels of the grid.}

\item{grid.labels.format}{The format of the labels of the grid.}

\item{grid.labels.cardinal}{The cardinal of the labels of the grid.}

\item{grid.labels.margin.x}{The x of the margin of the labels of the grid.}

\item{grid.labels.margin.y}{The y of the margin of the labels of the grid.}

\item{grid.labels.space.x}{The x of the space of the labels of the grid.}

\item{grid.labels.space.y}{The y of the space of the labels of the grid.}

\item{grid.labels.inside_frame}{The inside_frame of the labels of the grid.}

\item{grid.ticks}{The ticks of the grid.}

\item{grid.lines}{The lines of the grid.}

\item{grid.ndiscr}{The ndiscr of the grid.}

\item{mouse_coordinates.position}{The position of the mouse_coordinates. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{minimap.server}{The server of the minimap.}

\item{minimap.toggle}{The toggle of the minimap.}

\item{minimap.position}{The position of the minimap. A tm_pos object, or a shortcut of two values: horizontal (left, center, right) and vertical (top, center, bottom). See tm_pos for details}

\item{panel.show}{The visibility of the panel. \code{TRUE} or \code{FALSE}.}

\item{panel.labels}{The labels of the panel.}

\item{panel.label.size}{The size of the label of the panel.}

\item{panel.label.color}{The color of the label of the panel.}

\item{panel.label.fontface}{The font face of the label of the panel. See \code{graphics::par}, option 'font'.}

\item{panel.label.fontfamily}{The font family of the label of the panel. See \code{graphics::par}, option 'family'.}

\item{panel.label.alpha}{The alpha transparency of the label of the panel.}

\item{panel.label.bg}{The bg of the label of the panel.}

\item{panel.label.bg.color}{The color of the bg of the label of the panel.}

\item{panel.label.bg.alpha}{The alpha transparency of the bg of the label of the panel.}

\item{panel.label.frame}{The frame of the label of the panel.}

\item{panel.label.frame.color}{The color of the frame of the label of the panel.}

\item{panel.label.frame.alpha}{The alpha transparency of the frame of the label of the panel.}

\item{panel.label.frame.lwd}{The line width of the frame of the label of the panel. See \code{graphics::par}, option 'lwd'.}

\item{panel.label.frame.r}{The r (radius) of the frame of the label of the panel.}

\item{panel.label.height}{The height of the label of the panel.}

\item{panel.label.rot}{Rotation angles of the panel labels. Vector of four values that determine the panel label rotation when they are placed left, top, right, and bottom. The default angles are 90, 0, 270 and 0 respectively. Note that the second value is the most common, since labels are by default shown on top (see \code{panel.wrap.pos}). In cross-table facets created with \code{tm_facets_grid()}, the first two values are used by default (see \code{panel.xtab.pos}).}

\item{qtm.scalebar}{The scalebar of the qtm.}

\item{qtm.minimap}{The minimap of the qtm.}

\item{qtm.mouse_coordinates}{The mouse_coordinates of the qtm.}

\item{earth_boundary}{The earth boundary}

\item{earth_boundary.color}{The color of the earth_boundary.}

\item{earth_boundary.lwd}{The line width of the earth_boundary. See \code{graphics::par}, option 'lwd'.}

\item{earth_datum}{Earth datum}

\item{space}{Should the space be drawn? Only applicable is earth_boundary is enabled.}

\item{space.color}{The color of the space.}

\item{space_overlay}{Should the space be drawn as overlay (to make sure spatial features or rasters do not exceed the earth boundary), or as background? By default \code{TRUE} when a raster is warped.}

\item{check_and_fix}{Should attempt to fix an invalid shapefile}

\item{basemap.show}{The visibility of the basemap. \code{TRUE} or \code{FALSE}.}

\item{basemap.server}{The server of the basemap.}

\item{basemap.alpha}{The alpha transparency of the basemap.}

\item{basemap.zoom}{The zoom of the basemap.}

\item{tiles.show}{The visibility of the tiles. \code{TRUE} or \code{FALSE}.}

\item{tiles.server}{The server of the tiles.}

\item{tiles.alpha}{The alpha transparency of the tiles.}

\item{tiles.zoom}{The zoom of the tiles.}

\item{attr.color}{The color of the attr.}

\item{crs_extra}{Only used internally (work in progress)}

\item{crs_global}{The used crs for world maps}

\item{crs_basemap}{The crs_basemap (overall).}

\item{title}{deprecated See \code{\link[=tm_title]{tm_title()}}}

\item{main.title}{deprecated See \code{\link[=tm_title]{tm_title()}}}

\item{main.title.size, main.title.color, main.title.fontface, main.title.fontfamily, main.title.position}{deprecated. Use the \code{title.} options instead.}

\item{fontface, fontfamily}{renamed to \code{text.fontface} and \code{text.fontfamily}}

\item{style}{style see \code{tm_style()}}

\item{...}{List of tmap options to be set, or option names (characters) to be returned (see details)}
}
\description{
tmap options
}
\seealso{
\href{https://r-tmap.github.io/tmap/articles/basics_layout}{Layout}, \href{https://r-tmap.github.io/tmap/articles/adv_margins}{Margins and aspect ratio}, \href{https://r-tmap.github.io/tmap/articles/adv_options}{Options and styles}, \href{https://r-tmap.github.io/tmap/articles/examples_choro_World}{Choropleth (World)}, \href{https://r-tmap.github.io/tmap/articles/examples_terrain}{Terrain map}
}
